#!/usr/bin/env python

"""
Module contains classes presenting Element and Specie (Element + oxidation state).
"""

__author__="Shyue Ping Ong, Michael Kocher"
__copyright__ = "Copyright 2011, The Materials Project"
__version__ = "1.0"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__status__ = "Production"
__date__ ="$Sep 23, 2011M$"

import os
import re
import yaml

from pymatgen.core.design_patterns import singleton
from pymatgen.util.string_utils import formula_double_format

def _load__pt_data():
    """Loads element data from yaml file"""
    module_dir = os.path.dirname(os.path.abspath(__file__))
    with open(os.path.join(module_dir,"periodic_table.yaml")) as f:
        return yaml.load(f)

_pt_data  = _load__pt_data()
_pt_row_sizes = (2,8,8,18,18,32,32)

class Element(object):
    '''
    Basic immutable element object with all relevant properties.
    Only one instance of Element for each symbol is stored after creation, 
    ensuring that a particular element behaves like a singleton.
    '''
    
    _all_elements = {}

    def __new__(cls, *args):
        '''
        Prevents multiple Element objects from being created.  Faster and reduces memory usage.
        '''
        if tuple(args) in Element._all_elements:
            return Element._all_elements[tuple(args)]
        else:
            return super(Element, cls).__new__(cls)

    def __init__(self,symbol):
        '''
        Create immutable element from a symbol
        '''
        self._data = _pt_data[symbol]
        self._z = self._data['Atomic no']
        self._symbol = symbol
        
        Element._all_elements[tuple([symbol])] = self
        
    @property
    def Z(self):
        """Atomic number"""
        return self._z
    
    @property
    def symbol(self):
        """Element symbol"""
        return self._symbol
    
    @property
    def X(self):
        """Electronegativity"""
        if 'X' in self._data:
            return self._data['X']
        else:
            return 0
        
    @property
    def number(self):
        """Alternative attribute for atomic number"""
        return self.Z

    @property
    def name(self):
        """Full name for element"""
        return self._data['Name']
    
    @property
    def atomic_mass(self):
        """Atomic mass"""
        return self._data['Atomic mass']
    
    @property
    def atomic_radius(self):
        """Atomic radius"""
        return self._data['Atomic radius']
    
    @property
    def max_oxidation_state(self):
        """Maximum oxidation state for element"""
        return self._data['Max oxidation state']
    
    @property
    def min_oxidation_state(self):
        """Minimum oxidation state for element"""
        return self._data['Min oxidation state']
    
    @property
    def mendeleev_no(self):
        """Mendeleev number"""
        return self._data['Mendeleev no']
    
    @property
    def electrical_resistivity(self):
        """Electrical resistivity"""
        return self._data['Electrical resistivity']

    @property
    def velocity_of_sound(self):
        """Velocity of sound"""
        return self._data['Velocity of sound']

    @property
    def reflectivity(self):
        """Reflectivity"""
        return self._data['Reflectivity']
    
    @property
    def refractive_index(self):
        """Refractice index"""
        return self._data['Refractive index']

    @property
    def poissons_ratio(self):
        """Poisson's ratio"""
        return self._data['Poissons ratio']
    
    @property
    def molar_volume(self):
        """Molar volume"""
        return self._data['Molar volume']
    
    @property
    def electronic_structure(self):
        """Electronic structure. Simplified form with HTML formatting.
        E.g., The electronic structure for Fe is represented as [Ar].3d<sup>6</sup>.4s<sup>2</sup>
        """
        return self._data['Electronic structure']

    @property
    def full_electronic_structure(self):
        """
        Full electronic structure as tuple.
        E.g., The electronic structure for Fe is represented as:
        [(1, 's', 2), (2, 's', 2), (2, 'p', 6), (3, 's', 2), (3, 'p', 6), (3, 'd', 6), (4, 's', 2)]
        """
        estr = self._data['Electronic structure']
        def parse_orbital(orbstr):
            m = re.match("(\d+)([spdfg]+)<sup>(\d+)</sup>", orbstr)
            if m:
                return (int(m.group(1)), m.group(2), int(m.group(3)))
            return orbstr
        data = [parse_orbital(s) for s in estr.split(".")]
        if data[0][0] == "[":
            data = Element(data[0].replace("[", "").replace("]","")).full_electronic_structure + data[1:]
        return data
    
    @property
    def thermal_conductivity(self):
        """Thermal conductivity"""
        return self._data['Thermal conductivity']
    
    @property
    def boiling_point(self):
        """Boiling point"""
        return self._data['Boiling point']
    
    @property
    def melting_point(self):
        """Melting point"""
        return self._data['Melting point']
    
    @property
    def critical_temperature(self):
        """Critical temperature"""
        return self._data['Critical temperature']
    
    @property
    def superconduction_temperature(self):
        """Superconduction temperature"""
        return self._data['Superconduction temperature']
    
    @property
    def liquid_range(self):
        """Liquid range"""
        return self._data['Liquid range']

    @property
    def bulk_modulus(self):
        """Bulk modulus"""
        return self._data['Bulk modulus']
    
    @property
    def youngs_modulus(self):
        """Young's modulus"""
        return self._data['Youngs modulus']
    
    @property
    def brinell_hardness(self):
        """Brinell hardness"""
        return self._data['Brinell hardness']

    @property
    def rigidity_modulus(self):
        """Rigidity modulous"""
        return self._data['Rigidity modulus']
    
    @property
    def mineral_hardness(self):
        """Mineral hardness"""
        return self._data['Mineral hardness']
    
    @property
    def vickers_hardness(self):
        """Vicker's hardness"""
        return self._data['Vickers hardness']
    
    @property
    def density_of_solid(self):
        """Density of solid phase"""
        return self._data['Density of solid']

    @property
    def coefficient_of_linear_thermal_expansion(self):
        """Coefficient of linear thermal expansion"""
        return self._data['Coefficient of linear thermal expansion']

    def __eq__(self,other):
        if other == None:
            return False
        return self.Z == other.Z
    
    def __ne__(self,other):
        if other == None:
            return True
        return self.Z != other.Z

    def __hash__(self):
        return self.Z
    
    def __repr__(self):
        return "Element " + self.symbol

    def __str__(self):
        return self.symbol

    def __cmp__(self, other):
        '''
        Sets a default sort order for atomic species by electronegativity.  Very
        useful for getting correct formulas.  For example, FeO4PLi is automatically
        sorted in LiFePO4.
        '''
        return (self.X - other.X)

    @staticmethod       
    def from_Z(z):
        for sym in _pt_data.keys():
            if Element(sym).Z == z:
                return Element(sym)
        raise ValueError("No element with this atomic number")

    @staticmethod
    def from_row_and_group(row,group):
        """
        Returns an element from a row and group number.  Note that the 18 group number system is used, i.e. Noble gases are group 18.
        """
        for sym in _pt_data.keys():
            el = Element(sym)
            if el.row == row and el.group == group:
                return el
        return None

    @staticmethod
    def is_valid_symbol(symbol):
        """Returns true if symbol is a valid element symbol"""
        return symbol in _pt_data

    @property
    def row(self):
        """
        Returns the periodic table row of the element.
        """
        Z = self.Z
        totalEls = 0;
        if (Z >= 57 and Z <= 70):
            return 8
        elif (Z >= 89 and Z <= 102):
            return 9

        for i in xrange(len(_pt_row_sizes)):
            totalEls += _pt_row_sizes[i]
            if totalEls >= Z:
                return i+1
        return 8

    @property
    def group(self):
        """
        Returns the periodic table group of the element.
        """
        Z = self.Z
        if Z == 1:
            return 1
        if Z == 2:
            return 18
        if Z >= 3 and Z <= 18:
            if ((Z - 2) % 8 == 0):
                return 18
            elif ((Z - 2) % 8 <= 2):
                return (Z - 2) % 8;
            else:
                return (10 + (Z - 2) % 8)

        if Z >= 19 and Z <= 54:
            if ((Z - 18) % 18 == 0):
                return 18
            else:
                return (Z - 18) % 18

        if((Z - 54) % 32 == 0):
            return 18
        elif ((Z - 54) % 32 >= 17):
            return (Z - 54) % 32 - 14
        else:
            return (Z - 54) % 32

    @property
    def block(self):
        """docstring for block
        return the block character 's,p,d,f'
        """
        block = ''
        if self.group in [1,2]:
            block = 's'
        elif self.group in xrange(13,19):
            block = 'p'
        elif (self.is_actinoid() or self.is_lanthanid()):
            block = 'f'
        elif self.group in xrange(3,13):
            block = 'd'
        else:
            print("unable to determine block")
        return block

    @property
    def is_noble_gas(self):
        """
        True if element is noble gas.
        """
        ns = [2, 10, 18, 36, 54, 86, 118]
        return self.Z in ns

    @property
    def is_transition_metal(self):
        """
        True if element is a transition metal.
        """

        ns = range(21, 31)
        ns.extend(xrange(39, 49))
        ns.append(57)
        ns.extend(xrange(72, 81))
        ns.append(89)
        ns.extend(xrange(104, 113))
        return self.Z in ns
            
    @property
    def is_rare_earth_metal(self):
        """
        True if element is a rare earth metal.
        """
        return self.is_lanthanid or self.is_actinoid

    @property
    def is_metalloid(self):
        """
        True if element is a metalloid.
        """
        ns = ['B', 'Si', 'Ge', 'As', 'Sb', 'Te', 'Po']
        return self.symbol in ns
    
    @property
    def is_alkali(self):
        """
        True if element is an alkali metal.
        """
        ns = [3,11, 19,37, 55,87]
        return self.Z in ns
    
    @property
    def is_alkaline(self):
        """
        True if element is an alkaline earth metal (group II).
        """
        ns = [4, 12, 20, 38, 56, 88]
        return self.Z in ns
    
    @property
    def is_halogen(self):
        """
        True if element is a halogen.
        """
        ns = [9, 17, 35, 53, 85]
        return self.Z in ns

    @property
    def is_lanthanoid(self):
        """
        True if element is a lanthanoid.
        """
        return self.Z > 56 and self.Z < 72

    @property
    def is_actinoid(self):
        """
        True if element is a actinoid.
        """
        return self.Z > 88 and self.Z < 104


class Specie(object):
    """
    An extension of Element with an oxidation state.
    Note that while Specie does not directly inherit from Element 
    (because of certain implementation concerns due to the singleton nature of each element),
    it does inherit all Element attributes and hence function exactly as an Element would.
    """
           
    def __init__(self, symbol, oxidation_state):
        """
        Arguments:
            symbol - Element symbol, e.g., Fe
            oxidation_state - e.g., 2 or -2
        """
        self._el = Element(symbol)
        self._oxi_state = oxidation_state
        Element._all_elements[tuple([symbol, oxidation_state])] = self
    
    def __getattr__(self, attr):
        """
        Trick to make Specie inherit all Element properties.
        """
        if hasattr(self._el, attr):
            return getattr(self._el,attr) 
    
    def __eq__(self,other):
        """
        Specie is equal to other only if element and oxidation states are exactly the same.
        """
        if other == None:
            return False
        return self.Z == other.Z and self.oxi_state == other.oxi_state
    
    def __ne__(self,other):
        if other == None:
            return True
        return self.Z != other.Z or self.oxi_state != other.oxi_state

    def __hash__(self):
        """
        Given that all oxidation states are below 100 in absolute value, this should effectively ensure that no two unequal 
        Specie have the same hash.
        """
        return self.Z * 100 + self.oxi_state
    
    def __cmp__(self, other):
        '''
        Sets a default sort order for atomic species by electronegativity, followed by oxidation state. 
        '''
        return (self.X - other.X) * 100 + (self.oxi_state - other.oxi_state)
        
    @property
    def oxi_state(self):
        """
        Oxidation state.
        """
        return self._oxi_state
    
    @staticmethod
    def from_string(species_string):
        """
        Returns a Specie from a string representation. 
        For example, Mn2+, Fe3+, O2-.
        """
        m = re.search('([A-Z][a-z]*)([0-9\.]*)([\+\-])', species_string)
        if m:
            num = 1 if m.group(2) == "" else float(m.group(2))
            return Specie(m.group(1), num if m.group(3) == "+" else - num)
        else:
            raise ValueError("Invalid Species String")
    
    def __repr__(self):
        return "Specie " + self.__str__()
    
    def __str__(self):
        output = self.symbol
        if self.oxi_state >= 0:
            output += formula_double_format(self.oxi_state) + "+"
        else:
            output += formula_double_format(- self.oxi_state) + "-"
        return output

@singleton
class PeriodicTable(object):
    '''
    Periodic table singleton class.
    '''

    def __init__(self):
        """ Implementation of the singleton interface """
        self._all_elements = dict()
        for sym in _pt_data.keys():
            el = Element(sym)
            self._all_elements[sym] = el

    def __getattr__(self, name):
        return self._all_elements[name]
    
    def all_elements(self):
        return self._all_elements.values()

    @staticmethod
    def print_periodic_table(filter_function = None):
        for row in range(1,10):
            for group in range(1,19):
                el = Element.from_row_and_group(row,group)
                if el != None and ((not filter_function) or filter_function(el)):
                    print "%3s" % (el.symbol),
                else:
                    print "   ",
            print

def smart_element_or_specie(obj):
    """
    Utility method to get an Element or Specie from an input obj.
    If obj is in itself an element or a specie, it is returned automatically.
    If obj is an int, the Element with the atomic number obj is returned.
    If obj is a string, Specie parsing will be attempted (e.g., Mn2+), failing which Element parsing will be attempted (e.g., Mn).
    """
    if isinstance(obj,(Element, Specie)):
        return obj
    elif isinstance(obj,int):
        return Element.from_Z(obj)
    elif isinstance(obj,basestring):
        try:
            return Specie.from_string(obj)
        except ValueError:
            return Element(obj)
    raise ValueError("Can't parse Element or String from "+ str(obj))



#!/usr/bin/env python

"""
This module provides classes to perform fitting of two structures.
"""

from __future__ import division

__author__="Shyue Ping Ong, Geoffroy Hautier"
__copyright__ = "Copyright 2011, The Materials Project"
__version__ = "1.0"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__status__ = "Production"
__date__ ="$Sep 23, 2011M$"

import math
import itertools
import logging
from collections import OrderedDict

import numpy as np

from pymatgen.core.lattice import Lattice
from pymatgen.core.structure import Structure
from pymatgen.core.operations import SymmOp
from pymatgen.core.structure_modifier import StructureEditor

class StructureFitter(object):
    """
    Class to perform fitting of two structures
    """

    def __init__(self, structure_a, structure_b, tolerance_cell_misfit = 0.1, tolerance_atomic_misfit = 1.0, tol_partial_occup = 0.1, supercells_allowed = True, anonymized = False, order_disorder = False):
        """
        Fits two structures.
        All fitting parameters have been set with defaults that should work in most cases.
        To use, initialize the structure fitter with parameters, and then run fit()
        E.g.,
        fitter = StructureFitter(a, b)
        print fitter
        """
    
        self._tolerance_cell_misfit = tolerance_cell_misfit
        self._tolerance_atomic_misfit = tolerance_atomic_misfit
        self._tol_partial_occup = tol_partial_occup
        self._supercells_allowed = supercells_allowed
        self._anonymized = anonymized
        self._order_disorder = order_disorder
        #Sort structures first so that they have the same arrangement of species
        self._structure_a = structure_a.get_sorted_structure()
        self._structure_b = structure_b.get_sorted_structure()
        self._mapping_op = None
        if not self._anonymized:
            self.fit(self._structure_a, self._structure_b)
        else:
            comp_a = structure_a.composition
            comp_b = structure_b.composition
            if len(comp_a.elements) == len(comp_b.elements):
                el_a = comp_a.elements
                #Create permutations of the specie/elements in structure A 
                for p in itertools.permutations(el_a):
                    # Create mapping of the specie/elements in structure B to that of A.
                    # Then create a modified structure with those elements and try to fit it.
                    el_mapping = dict(zip(comp_b.elements, p))
                    logging.info("Using specie mapping " + str(el_mapping))
                    mod = StructureEditor(self._structure_b)
                    mod.replace_species(el_mapping)
                    self.fit(self._structure_a, mod.modified_structure)
                    if self._mapping_op != None:
                        #Store successful element mapping
                        self.el_mapping = {el_a:el_b for el_b, el_a in el_mapping.items()}
                        break                
            else:
                logging.info("No. of elements in structures are unequal.  Cannot be fitted!")          

    def fit(self, a, b): 
        """
        Compares two structures and give the possible affine mapping that
        transforms one into another.
        """
        biggest_dist = 0
        logging.info("Structure a")
        logging.info(str(a))
        logging.info("Structure b")
        logging.info(str(b))
        
        # Check composition first.  If compositions are not the same, do not need to fit further.
        if a.composition.reduced_formula != b.composition.reduced_formula or ((a.num_sites != b.num_sites) and not self._supercells_allowed):
            logging.info('Compositions do not match')
            return None
        
        logging.info('Compositions match')

        # Fitting is done by matching sites in one structure (to_fit)
        # to the other (fixed).  
        # We set the structure with fewer sites as fixed, 
        # and scale the structures to the same density
        (fixed, to_fit)  = self._scale_structures(a,b)
        
        # Defines the atom misfit tolerance
        tol_atoms = self._tolerance_atomic_misfit * ( 3 * 0.7405 * fixed.volume / (4 * math.pi * fixed.num_sites)) ** (1 / 3)
        logging.info("Atomic misfit tolerance = %.4f" % (tol_atoms) )
        
        max_sites = 1e100
        # determine which type of sites to use for the mapping
        for sp in to_fit.species_and_occu:
            sp_sites = [site for site in to_fit if site.species_and_occu == sp]
            if len(sp_sites) < max_sites:
                fit_sites = sp_sites
                max_sites = len(sp_sites) 
       
        # Set the arbitrary origin
        origin = fit_sites[0]
        logging.info("Origin = " + str(origin) )
        
        #Get candidate rotations
        cand_rot = self._get_candidate_rotations(origin, fixed, to_fit)
        
        logging.info(" FOUND " + str(len(cand_rot)) + " candidate rotations ")
        if len(cand_rot) == 0:
            logging.info("No candidate rotations found, returning null. ")
            return None
        
        # now that candidate rotations have been found, shift origin of to_fit
        # because the symmetry operations will be applied from this origin
        oshift = SymmOp.from_rotation_matrix_and_translation_vector(np.eye(3), -origin.coords)
        to_fit = apply_operation(to_fit, oshift)
        # sort the operations, the first ones are the ones with small shear
        # this assures us that we find the smallest cell misfit fits
        sorted_cand_rot = sorted(cand_rot.keys(), key=lambda r: cand_rot[r])

        tol_atoms_plus = 1.1 * tol_atoms
        found_map = False
        mapping_op = None
        
        for rot in sorted_cand_rot:
            for site in fixed:
                logging.info("Trying candidate rotation : \n" + str(rot))
                if site.species_and_occu == origin.species_and_occu:
                    shift = site.coords
                    op = SymmOp.from_rotation_matrix_and_translation_vector(rot.rotation_matrix, shift)
                    nstruct = apply_operation(to_fit, op)
                    correspondance = OrderedDict()
                    all_match = True
                    biggest_dist = 0
                    # check to see if transformed struct matches fixed structure
                    for trans in nstruct:
                        cands = fixed.get_sites_in_sphere(trans.coords, tol_atoms_plus)
                        if len(cands) == 0:
                            logging.info("No candidates found1")
                            all_match = False
                            break
                        cands = sorted(cands, key = lambda a: a[1])
                        (closest, closest_dist) = cands[0]
                        if closest_dist > tol_atoms or closest.species_and_occu != trans.species_and_occu:
                            logging.info("Closest dist too large! closest dist = " + str(closest_dist))
                            all_match = False
                            break
                        correspondance[trans] = closest
                        if closest_dist > biggest_dist:
                            biggest_dist = closest_dist
                    
                    if not all_match:
                        continue
                    
                    if not are_sites_unique(correspondance.values(), False):
                        all_match = False
                    else:
                        for k,v in correspondance.items():
                            logging.info(str(k) + " fits on " + str(v))
                        
                    # now check to see if the converse is true -- do all of the
                    # sites of fixed match up with a site in toFit
                    # this used to not be here. This fixes a bug.
                    logging.info("Checking inverse mapping")
                    inv_correspondance = OrderedDict()
    
                    # it used to be fixed.getNumSites() != nStruct.getNumSites()
                    # so only when the number of sites are different but it's
                    # actually better to allways check the reverse. This
                    # elimininates weird situations where two atoms fit to one (reduced in the
                    # unit cell)
                    for fixed_site in fixed:
                        cands = nstruct.get_sites_in_sphere(fixed_site.coords, tol_atoms_plus)
                        if len(cands) == 0: 
                            logging.info("Rejected because inverse mapping does not fit - Step 1")
                            all_match = False
                            break
                        
                        cands = sorted(cands, key = lambda a: a[1])
                        (closest, closest_dist) = cands[0]
                        
                        if closest_dist > tol_atoms or closest.species_and_occu != fixed_site.species_and_occu:
                            all_match = False
                            logging.info("Rejected because inverse mapping does not fit - Step 2")
                            break
                        inv_correspondance[fixed_site] = closest
                    
                    if all_match:
                        if not are_sites_unique(inv_correspondance.values(), False):
                            all_match = False
                            logging.info("Rejected because two atoms fit to the same site for the inverse")
                    
                    if all_match:
                        self.inv_correspondance = inv_correspondance
                        logging.info("Correspondance for the inverse")
                        for k,v in inv_correspondance.items():
                            logging.info(str(k) + " fits on " + str(v))
                            
                    # The smallest correspondance array shouldn't have any equivalent sites
                    if fixed.num_sites != to_fit.num_sites:
                        logging.info("Testing sites unique")
                        if not are_sites_unique(correspondance.values()): 
                            all_match = False
                            logging.info("Rejected because the smallest correspondance array has equivallent sites")
                            break
        
                    if all_match:
                        found_map = True
                        mapping_op = op
                        self.correspondance = correspondance
                        break
            
            if found_map:
                break
        
        logging.info("Done testing all candidate rotations")
        
        self._atomic_misfit = biggest_dist / ((3 * 0.7405 * a.volume / (4 * math.pi * a.num_sites)) ** (1/3))
        if mapping_op != None:
            rot = mapping_op.rotation_matrix # maps toFit to fixed
            p = sqrt_matrix(np.dot(rot.transpose(),rot))
            scale_matrix = np.eye(3) * self.scale
            newrot = np.dot(scale_matrix, rot)
            # we need to now make sure fitterdata.MappingOp maps b -> a and not
            # the other way around
            mshift = np.dot(rot, oshift.translation_vector)
            finaltranslation = mapping_op.translation_vector + mshift[0]
            composite_op = SymmOp.from_rotation_matrix_and_translation_vector(newrot, finaltranslation)
            self._mapping_op = composite_op if self.fixed_is_a else composite_op.inverse
            #self.time = System.currentTimeMillis() - start
            self._cell_misfit = shear_invariant(p)
        

    def __str__(self):
        
        output = ["Fitting structures"]
        output.append("\nStructure 1:")
        output.append(str(self._structure_a))
        output.append("\nStructure 2:")
        output.append(str(self._structure_b))
        output.append("\nFitting parameters:")
        output.append("\tTolerance cell misfit = " + str(self._tolerance_cell_misfit))
        output.append("\tTol partial occup = " + str(self._tol_partial_occup))
        output.append("\tSupercells allowed = " + str(self._supercells_allowed))
        output.append("\tAnonymized  = " + str(self._anonymized))
        output.append("\tOrder disorder  = " + str(self._order_disorder))
        output.append("\nFitting " + ("succeeded " if self._mapping_op != None else "failed"))
        if self._mapping_op != None:
            output.append("\tMapping op = " + str(self._mapping_op))
            output.append("\tCell misfit = " + str(self._cell_misfit))
            output.append("\tAtomic misfit = " + str(self._atomic_misfit))
            if self._anonymized:
                output.append("Element mapping")
                output.append("\n".join([str(k) + " -> " + str(v) for k,v in self.el_mapping.items()]))  
        return "\n".join(output)

    def _scale_structures(self, a, b):
        # which structure do we want to fit to the other ?
        # assume that structure b has less sites and switch if needed
        self.fixed_is_a = a.num_sites > b.num_sites
        (fixed, to_fit) = (a, b) if self.fixed_is_a else (b, a)
            
        # scale the structures to the same density
        rho_a = fixed.num_sites / fixed.volume
        rho_b = to_fit.num_sites / to_fit.volume
        scale = (rho_b / rho_a) ** (1/3)
        self.scale = scale
        to_fit = Structure(Lattice(to_fit.lattice.matrix * scale), to_fit.species_and_occu, to_fit.frac_coords)
        return (fixed, to_fit)
        
    
    def _get_candidate_rotations(self, origin, fixed, to_fit):
        tol_shear = self._tolerance_cell_misfit
        fixed_basis = fixed.lattice.matrix.transpose()
        # need to generate candidate rotations ...
        lengths = fixed.lattice.abc
        
        shells = []
        for i in range(3):
            dr = lengths[i] * math.sqrt(tol_shear / 2)
            shell = to_fit.get_neighbors_in_shell(origin.coords, lengths[i], dr)
            logging.info("shell " + str(i) + " radius=[" + str(lengths[i]) + "] dr=[" + str(dr) + "]")
            shells.append([site for (site, dist) in shell if site.species_and_occu == origin.species_and_occu])
            logging.info("No. in shell = " + str(len(shells[-1])))
        # now generate candidate rotations
        cand_rot = {} #Dict of SymmOp : float
        
        for pool in itertools.product(*shells):
            if all([nn.species_and_occu == origin.species_and_occu for nn in pool]):
                # now, can a unitary transformation bring the cell vectors together
                cell_v = np.array([nn.coords - origin.coords for nn in pool]).transpose()
                det = np.linalg.det(cell_v)
                if abs(det) < 0.001 or abs(abs(det) - fixed.volume) > 0.01:
                    continue
                rot = np.dot(fixed_basis, np.linalg.inv(cell_v))
                r = SymmOp.from_rotation_matrix_and_translation_vector(rot, np.array([0,0,0]))
    
                if r not in cand_rot:
                    transf = r.rotation_matrix
                    transf = np.dot(transf.transpose(), transf)
                    transf = np.eye(3) if almost_identity(transf) else transf
                    pbis = sqrt_matrix(transf)
                    if shear_invariant(pbis) < tol_shear:
                        cand_rot[r] = shear_invariant(pbis)
        
        return cand_rot

    @property
    def fit_found(self):
        return self.mapping_op != None

    @property
    def mapping_op(self):           
        return self._mapping_op

    @property
    def structure_a(self):
        return self._structure_a

    @property
    def structure_b(self):
        return self._structure_b

def apply_operation(structure, symmop):
    editor = StructureEditor(structure)
    editor.apply_operation(symmop)
    return editor.modified_structure

def sqrt_matrix(input_matrix):
    d,v = np.linalg.eig(input_matrix)
    diagonalbis = np.array([[d[0]**0.5,0,0],[0,d[1]**0.5,0],[0,0,d[2]**0.5]]) 
    temp = np.dot(diagonalbis,v.transpose())
    result = np.dot(v, temp)
    return result

def shear_invariant(matrix):
    return (matrix[0][0]-matrix[1][1]) ** 2 + (matrix[1][1]-matrix[2][2]) ** 2 + (matrix[0][0]-matrix[2][2]) ** 2 + 6*(matrix[0][1]*matrix[0][1]+matrix[0][2]*matrix[0][2]+matrix[1][2]*matrix[1][2])

def are_sites_unique(sites, allow_periodic_image = True):
    for (site1, site2) in itertools.combinations(sites,2):
        if (allow_periodic_image and site1.is_periodic_image(site2)) or (site1.species_and_occu == site2.species_and_occu and (abs(site1.coords - site2.coords) < 0.1).all()):
            return False
    return True

def closest_site_to_point(pt, list_of_sites):
    closest_dist = 1e10
    for c in list_of_sites:
        dist = np.linalg.norm(c.coords - pt)       
        if dist < closest_dist:
            closest = c
            closest_dist = dist
    return (closest, closest_dist)

def almost_identity(mat):
    """
    This is to resolve a very very strange bug in numpy that causes random eigen vectors to be returned
    for matrices very very close to the identity matrix.  See test_eig for examples.
    """
    return (abs(mat-np.eye(3)) < 1e-10).all()



from setuptools import setup, find_packages

setup (
  name = 'pymatgen',
  version = '1.0.4',
  packages = find_packages(),
  install_requires = ['numpy', 'scipy', 'matplotlib', 'PyCIFRW', 'PyYAML'],

  data_files=[('data', ['pymatgen/core/periodic_table.yaml']),
              ('config', ['pymatgen/io/VaspParameterSets.cfg'])],
  author = 'Shyue Ping Ong, Anubhav Jain, Michael Kocher, Dan Gunter',
  author_email = 'shyue@mit.edu, anubhavj@mit.edu, mpkocher@lbnl.gov, dkgunter@lbl.gov',
  maintainer = 'Shyue Ping Ong',
  url = 'https://github.com/CederGroupMIT/pymatgen_repo/',
  license = 'MIT',
  long_description = 'pymatgen is a Python library for the Materials Project (www.materialsproject.org). It includes core structure definition and utilities, electronic structure objects, and convenient IO from VASP and CIF files.',
  keywords = ["vasp", "materials", "project", "electronic", "structure"],
  classifiers = [
        "Programming Language :: Python :: 2.7",
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering :: Information Analysis",
        "Topic :: Scientific/Engineering :: Physics",
        "Topic :: Scientific/Engineering :: Chemistry",
        "Topic :: Software Development :: Libraries :: Python Modules",
  ],
  download_url = "https://github.com/CederGroupMIT/pymatgen_repo/tarball/master",
)




#!/usr/bin/env python

"""
This module provides classes used to define a structure, such as 
Site, PeriodicSite, Structure, and Composition.
"""

from __future__ import division

__author__="Shyue Ping Ong"
__copyright__ = "Copyright 2011, The Materials Project"
__version__ = "1.0"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__status__ = "Production"
__date__ ="$Sep 23, 2011M$"

import re
import math
import collections
import itertools
from fractions import gcd

import numpy as np
from pymatgen.core.lattice import Lattice
from pymatgen.core.periodic_table import Element, Specie, smart_element_or_specie
from pymatgen.util.string_utils import formula_double_format

class Site(collections.Mapping, collections.Hashable):
    '''
    A site object.  Atoms and occupancies should be a dictionary of element:occupancy
    or an element, in which case the occupancy default to 1.
    Vacancies are simply denoted with a None species.
    if the supplied atom_n_occu sum to less than 1, the remaining is automatically
    allocated as vacancy. e.g., a Site({"Si":0.5},coords) is basically a site which
    has a 0.5 occupancy of Si and 0.5 occupancy of vacancy.
    Coords are given in standard cartesian coordinates, NOT fractional coords.
    If you want to create a Site from a fractional coord, use the static constructor
    from_frac_coords instead.
    '''
    
    def __init__(self, atoms_n_occu, coords):
        """
        Create a periodic site.
        Arguments:
            atoms_n_occu - dict of elements or species and occupancies. Elements can be specified as 
                           symbols (Fe), atomic numbers (27), or actual Element objects.  Specie can
                           be specified as Specie objects, or strings (Fe2+).
            coords - Cartesian coordinates of site
        """
        
        if isinstance(atoms_n_occu,dict):
            self._species = {smart_element_or_specie(k) : v for k, v in atoms_n_occu.items()}
            totaloccu = sum(self._species.values())
            if totaloccu > 1:
                raise ValueError("Species occupancies sum to more than 1!")
            self._is_ordered = (totaloccu == 1 and len(self._species) == 1)
        else:
            self._species = {smart_element_or_specie(atoms_n_occu):1}
            self._is_ordered = True
        self._coords = coords
        
    def distance(self,other):
        """
        Get distance between two sites.
        """
        return np.linalg.norm(other.coords-self.coords)

    def distance_from_point(self,pt):
        """
        Returns distance between the site and a point in space.
        """
        return np.linalg.norm(pt-self._coords)

    @property
    def species_string(self):
        if self._is_ordered:
            return str(self._species.keys()[0])
        else:
            return ', '.join(["%s: %.4f" % (str(atom),occu) for atom,occu in self._species.items()])

    @property
    def species_and_occu(self):
        """
        The species at the site, i.e. a dict of element and occupancy
        """
        return self._species

    @property
    def specie(self):
        """
        The Specie/Element at the site. Only works for Ordered Sites.  Otherwise an AttributeError is raised.
        Use this property sparingly.  Robust design should make use of the property species_and_occu instead.
        """
        if not self._is_ordered:
            raise AttributeError("specie property only works for ordered sites!")
        return self._species.keys()[0]
   
    @property
    def coords(self):
        """
        A copy of the cartesian coordinates of the site as a numpy array.
        """
        return np.copy(self._coords)
    
    @property
    def is_ordered(self):
        return self._is_ordered

    @property 
    def x(self):
        """
        Cartesian x coordinate
        """
        return self._coords[0]

    @property
    def y(self):
        """
        Cartesian y coordinate
        """
        return self._coords[1]

    @property 
    def z(self):
        """
        Cartesian z coordinate
        """
        return self._coords[2]
    
    def __getitem__(self,el):
        '''
        Get the occupancy for element
        '''
        return self._species[el]
    
    def __eq__(self, other):
        if other == None:
            return False
        return self._species == other._species and np.allclose(self._coords, other._coords)
    
    def __ne__(self, other):
        return not self.__eq__(other)
    
    def __hash__(self):
        '''
        Minimally effective hash function that just distinguishes between Sites with different elements.
        '''
        hashcode = 0
        for el in self._species.keys():
            #Ignore elements with zero amounts.
            if self[el] != 0:
                hashcode += el.Z 
        return hashcode
    
    def __contains__(self,el):
        return el in self._species
    
    def __len__(self):
        return len(self._species)
    
    def __iter__(self):
        return self._species.__iter__()
    
    def __repr__(self):
        outs = []
        outs.append("Non-periodic Site")
        outs.append("xyz        : (%0.4f, %0.4f, %0.4f)" % tuple(self.coords))
        for k,v in self._species.items():
            outs.append("element    : %s" % k.symbol)
            outs.append("occupation : %0.2f" % v)
        return "\n".join(outs)

    def __cmp__(self, other):
        '''
        Sets a default sort order for atomic species by electronegativity.  Very
        useful for getting correct formulas.  For example, FeO4PLi is automatically
        sorted in LiFePO4.
        '''
        def avg_electroneg(sps):
            return sum([sp.X * occu for sp, occu in sps.items()])
        
        if avg_electroneg(self._species) < avg_electroneg(other._species):
            return -1
        if avg_electroneg(self._species) > avg_electroneg(other._species):
            return 1
        return 0

    def __str__(self):
        return "%s %s" %(self._coords, ','.join(["%s %.4f" % (str(atom),occu) for atom,occu in self._species.items()]))

    @property
    def to_dict(self):
        species_list = []
        for spec, occu in self._species.items():
            if isinstance(spec, Specie):
                species_list.append({'element': spec.symbol, 'occu': occu, 'oxidation_state': spec.oxi_state})
            elif isinstance(spec, Element):
                species_list.append({'element': spec.symbol, 'occu': occu})
        return {'name': self.species_string, 'species': species_list, 'occu': occu, 'xyz':list(self._coords)}

class PeriodicSite(Site):
    """
    Extension of generic Site object to periodic systems.
    PeriodicSite includes a lattice system.
    """

    def __init__(self, atoms_n_occu, coords, lattice, to_unit_cell=False, coords_are_cartesian = False):
        """
        Create a periodic site.
        Arguments:
            atoms_n_occu - dict of elements and occupancies
            frac_coords - coordinates of site in fractional coordinate of lattice
            lattice - a pymatgen.core.lattice Lattice associated with the site
            to_unit_cell - transforms fractional coordinate to the basic unit cell, i.e. 
                         all fractional coordinates satisfy 0 <= a < 1 
        """
        self._lattice = lattice
        self._fcoords = lattice.get_fractional_coords(coords) if coords_are_cartesian else coords
        
        if to_unit_cell:
            for i in xrange(len(self._fcoords)):
                self._fcoords[i] = self._fcoords[i] - math.floor(self._fcoords[i]) 
                
        c_coords = lattice.get_cartesian_coords(self._fcoords)
        Site.__init__(self,atoms_n_occu,c_coords)

    @property
    def lattice(self):
        """
        The lattice associated with the site.
        """
        return self._lattice

    @property
    def frac_coords(self):
        """
        The fractional coordinates of the site as a tuple.
        """
        return np.copy(self._fcoords)
    
    @property
    def a(self):
        """
        Fractional a coordinate
        """
        return self._fcoords[0]

    @property
    def b(self):
        """
        Fractional b coordinate
        """
        return self._fcoords[1]

    @property
    def c(self):
        """
        Fractional c coordinate
        """
        return self._fcoords[2]
    
    @property
    def to_unit_cell(self):
        fcoords = [i - math.floor(i) for i in self._fcoords]
        return PeriodicSite(self._species, fcoords, self._lattice)
    
    def is_periodic_image(self, other, tolerance = 1e-8):
        """
        Returns True if sites are periodic images of each other
        """
        if self.lattice != other.lattice:
            return False
        return  (abs(self._fcoords - other._fcoords) % 1 < tolerance).all()
    
    def __eq__(self, other):
        return self._species == other._species and self._lattice == other._lattice and np.allclose(self._coords, other._coords)
    
    def __ne__(self, other):
        return not self.__eq__(other)
    
    def distance_and_image_old(self,other,jimage=None):
        """
        Get distance between two sites assuming periodic boundary conditions.
        If the index jimage of two sites atom j is not specified it selects the j image nearest to the i atom
        and returns the distance and jimage indices in terms of lattice vector translations.
        if the index jimage of atom j is specified it returns the distance between
        the i atom and the specified jimage atom, the given jimage is also returned.
        Arguments:
            other - other site to get distance from.
            jimage - specific periodic image in terms of lattice translations, 
                     e.g., [1,0,0] implies to take periodic image that is one a-lattice vector away.
                     if jimage == None, the image that is nearest to the site is found.
        Return:
            (distance, jimage) - distance and periodic lattice translations of the other site
                                 for which the distance applies.
        
        COMMENT
        assumes the primitive cell vectors are sufficiently not skewed such that the condition
        |a|cos(ab_angle) < |b| for all possible cell vector pairs
        **** this method does not check this condition ****
        
        Shyue - THis code is kept for information reasons. A new version has been written which
        does not suffer the limitation above, but at a higher computational cost.
        """
        if jimage == None:
            #Old algorithm
            jimage = - np.array(np.around(other._fcoords-self._fcoords),int)
        return np.linalg.norm(self.lattice.get_cartesian_coords(jimage + other._fcoords - self._fcoords)), jimage
    
    def distance_and_image_from_frac_coords(self, fcoords, jimage=None):
        """
        Get distance between site and a fractional coord assuming periodic boundary conditions.
        If the index jimage of two sites atom j is not specified it selects the j image nearest to the i atom
        and returns the distance and jimage indices in terms of lattice vector translations.
        if the index jimage of atom j is specified it returns the distance between
        the i atom and the specified jimage atom, the given jimage is also returned.
        Arguments:
            other - other site to get distance from.
            jimage - specific periodic image in terms of lattice translations, 
                     e.g., [1,0,0] implies to take periodic image that is one a-lattice vector away.
                     if jimage == None, the image that is nearest to the site is found.
        Return:
            (distance, jimage) - distance and periodic lattice translations of the other site
                                 for which the distance applies.
        
        COMMENT
        Shyue - This slower code fixes the old problem with skewed cells.  
        """
        if jimage == None:
            adj1 = np.array([- math.floor(i) for i in self._fcoords])
            adj2 = np.array([- math.floor(i) for i in fcoords])
            mindist = 1e100
            coord1 = self._fcoords + adj1
            coord2 = fcoords + adj2
            test_set = [[-1,0] if coord1[i] < coord2[i] else [0,1] for i in xrange(3)]
            for image in itertools.product(*test_set):
                dist = np.linalg.norm(self.lattice.get_cartesian_coords(coord2 + image - coord1))
                if dist < mindist:
                    mindist = dist
                    jimage = adj2 - adj1 + image
            return mindist, jimage        
        return np.linalg.norm(self.lattice.get_cartesian_coords(jimage + fcoords - self._fcoords)), jimage
    
    def distance_and_image(self,other,jimage=None):
        """
        Get distance between two sites assuming periodic boundary conditions.
        """
        return self.distance_and_image_from_frac_coords(other._fcoords, jimage)

    def distance(self,other,jimage=None):
        """
        Get distance between two sites assuming periodic boundary conditions.
        Arguments:
            other - other site to get distance from.
            jimage - specific periodic image in terms of lattice translations, 
                     e.g., [1,0,0] implies to take periodic image that is one a-lattice vector away.
                     if jimage == None, the image that is nearest to the site is found.
        Return:
            distance - distance between the two sites
        
        """
        return self.distance_and_image(other, jimage)[0]

    def distance_from_point(self,pt):
        return np.linalg.norm(pt - self.coords)
    
    def __repr__(self):
        outs = []
        outs.append("Periodic Site")
        outs.append("abc : (%0.4f, %0.4f, %0.4f)" % tuple(self._fcoords))
        for k,v in self._species.items():
            outs.append("element    : %s" % k.symbol)
            outs.append("occupation : %0.2f" % v)
        return "\n".join(outs)

    @property
    def to_dict(self):
        species_list = []
        for spec, occu in self._species.items():
            if isinstance(spec, Specie):
                species_list.append({'element': spec.symbol, 'occu': occu, 'oxidation_state': spec.oxi_state})
            elif isinstance(spec, Element):
                species_list.append({'element': spec.symbol, 'occu': occu})
        return {'label': self.species_string, 'species': species_list, 'occu': occu, 'xyz':list(self._coords), 'abc':list(self._fcoords)}

class Structure(collections.Sequence, collections.Hashable):
    """
    Basic Structure object with periodicity. Essentially a sequence of sites having a common lattice.
    Structure is made to be immutable so that they can function as keys in a dictionary.  
    Modifications should be done by making a new Structure.
    """
    
    DISTANCE_TOLERANCE = 0.01
    
    def __init__(self, lattice, atomicspecies, coords, validate_proximity = False, to_unit_cell = False, coords_are_cartesian = False):
        """
        Create a periodic structure.
        Arguments:
            lattice - pymatgen.core.lattice Lattice object signify the lattice.
            atomicspecies- list of atomic species.  dict of elements and occupancies.
            fractional_coords - list of fractional coordinates of each species.
            validate_proximity - Whether to check if there are sites that are less than 1 Ang apart. Defaults to false.
            coords_are_cartesian - Set to True if you are providing coordinates in cartesian coordinates. Defaults to false.
        """
        if len(atomicspecies) != len(coords):
            raise StructureError("The list of atomic species must be of the same length as the list of fractional coordinates.")
           
        if isinstance(lattice,Lattice):
            self._lattice = lattice
        else:
            self._lattice = Lattice(lattice)
        
        self._sites = [PeriodicSite(atomicspecies[i],coords[i],self._lattice, to_unit_cell, coords_are_cartesian) for i in xrange(len(atomicspecies))]
        if validate_proximity:
            for (s1, s2) in itertools.combinations(self._sites, 2):
                if s1.distance(s2) < Structure.DISTANCE_TOLERANCE:
                    raise StructureError("Structure contains sites that are less than 0.01 Angstrom apart!")
    
    @property
    def lattice(self):
        """
        Lattice of the structure.
        """
        return self._lattice
    
    @property
    def species(self):
        """
        List of species at each site of the structure.
        Only works for ordered structures.
        Disordered structures will raise an AttributeError.
        """
        return [site.specie for site in self._sites]
    
    @property
    def species_and_occu(self):
        """
        List of species and occupancies at each site of the structure.
        """
        return [site.species_and_occu for site in self._sites]
    
    @property
    def sites(self):
        """
        Returns an iterator for the sites in the Structure. 
        """
        for site in self._sites:
            yield site

    def __contains__(self, site):
        return site in self._sites
    
    def __iter__(self):
        return self._sites.__iter__()

    def __getitem__(self, ind):
        return self._sites[ind]

    def __len__(self):
        return len(self._sites)
    
    def __eq__(self, other):
        if other == None:
            return False
        if len(self) != len(other):
            return False
        for site in self:
            if site not in other:
                return False
        return True
    
    def __ne__(self, other):
        return not self.__eq__(other)
    
    def __hash__(self):
        #for now, just use the composition hash code.
        return self.composition.__hash__()

    @property
    def num_sites(self):
        """
        Number of sites in the structure.
        """
        return len(self._sites)
    
    @property
    def frac_coords(self):
        '''
        Returns the fractional coordinates.
        '''
        return [site.frac_coords for site in self._sites]

    @property
    def cart_coords(self):
        '''
        Returns a list of the cartesian coordinates of sites in the structure.
        '''
        return [site.coords for site in self._sites]

    @property
    def volume(self):
        '''
        Returns the volume of the structure.
        '''
        return self._lattice.volume
   
    @property
    def density(self):
        '''
        Returns the density in units of g/cc
        '''
        constant = 1.660468
        return self.composition.weight/self.volume*constant
    
    def get_distance(self,i,j,jimage=None):
        """
        get distance between site i and j assuming periodic boundary conditions
        if the index jimage of two sites atom j is not specified it selects the j image nearest to the i atom
        and returns the distance and jimage indices in terms of lattice vector translations
        if the index jimage of atom j is specified it returns the distance between
        the i atom and the specified jimage atom, the given jimage is also returned.
        """
        return self[i].distance(self[j], jimage)
    
    def get_sites_in_sphere(self, pt, r):
        '''
        Find all sites within a sphere from the structure. This includes sites in other periodic images.
        Algorithm: 
        (1) place sphere of radius r in crystal and determine minimum supercell (parallelpiped) which would
            contain a sphere of radius r. for this we need the projection of a_1 on a unit vector perpendicular 
            to a_2 & a_3 (i.e. the unit vector in the direction b_1) to determine how many a_1's it will
            take to contain the sphere. 
                  Nxmax = r * length_of_b_1 / (2 Pi)
        (2) keep points falling within r.
        The return type is a [(site, dist) ...] since most of the time, subsequent 
        processing requires the distance.
        '''
        recp_len = self._lattice.reciprocal_lattice.abc
        sr = r + 0.15
        nmax = [sr * l / (2 * math.pi) for l in recp_len]
        pcoords = self._lattice.get_fractional_coords(pt)
        nxmax = int(math.floor(pcoords[0] + nmax[0]))
        nxmin = int(math.floor(pcoords[0] - nmax[0]))
        nymax = int(math.floor(pcoords[1] + nmax[1]))
        nymin = int(math.floor(pcoords[1] - nmax[1]))
        nzmax = int(math.floor(pcoords[2] + nmax[2]))
        nzmin = int(math.floor(pcoords[2] - nmax[2]))
        neighbors = []
        n = len(self._sites)
        site_fcoords = np.array([site.to_unit_cell.frac_coords for site in self._sites])
        pts = [pt] * n
        for image in itertools.product(xrange(nxmin, nxmax+1), xrange(nymin, nymax+1), xrange(nzmin, nzmax+1)):
            submat = [image] * n
            fcoords = site_fcoords + submat
            coords = self._lattice.get_cartesian_coords(fcoords)
            dists = (coords - pts) ** 2
            dists = np.sqrt(dists.sum(axis = 1))
            withindists = (dists <= r)
            for i in xrange(n):
                if withindists[i]:
                    neighbors.append((PeriodicSite(self._sites[i].species_and_occu, fcoords[i], self._lattice), dists[i]))
        
        return neighbors
    
    
    def get_neighbors(self, site, r):
        return [(s, dist) for (s, dist) in self.get_sites_in_sphere(site.coords, r) if site != s]
    
    def get_all_neighbors(self, r):
        """
        Get neighbors for each atom in the unit cell, out to a distance r
        Returns a list of list of neighbors for each site in structure.
        Use this method if you are planning on looping over all sites in the
        crystal. If you only want neighbors for a particular site, use the method
        get_neighbors as it may not have to build such a large supercell
        However if you are looping over all sites in the crystal, this method is
        more efficient since it only performs *one* pass over a large enough 
        supercell to contain all possible atoms out to a distance r.
        The return type is a [(site, dist) ...] since most of the time, subsequent 
        processing requires the distance.
        """
    
        #use same algorithm as getAtomsInSphere to determine supercell but
        #loop over all atoms in crystal
        recp_len = self.lattice.reciprocal_lattice.abc
        sr = r + 0.15
        nmax = [sr * l / (2 * math.pi) for l in recp_len]
        site_nminmax = []
        unit_cell_sites = [site.to_unit_cell for site in self._sites]
        #unit_cell_coords = [site.frac_coords for site in unit_cell_sites]
        
        for site in self._sites:
            pcoords = site.frac_coords
            inxmax = int(math.floor(pcoords[0] + nmax[0]))
            inxmin = int(math.floor(pcoords[0] - nmax[0]))
            inymax = int(math.floor(pcoords[1] + nmax[1]))
            inymin = int(math.floor(pcoords[1] - nmax[1]))
            inzmax = int(math.floor(pcoords[2] + nmax[2]))
            inzmin = int(math.floor(pcoords[2] - nmax[2]))
            site_nminmax.append(((inxmin, inxmax), (inymin, inymax), (inzmin, inzmax)))

        nxmin = min([i[0][0] for i in site_nminmax])
        nxmax = max([i[0][1] for i in site_nminmax])
        nymin = min([i[1][0] for i in site_nminmax])
        nymax = max([i[1][1] for i in site_nminmax])
        nzmin = min([i[2][0] for i in site_nminmax])
        nzmax = max([i[2][1] for i in site_nminmax])
        
        neighbors = [list() for i in range(len(self._sites))]
        
        site_coords = np.array(self.cart_coords)
        n = len(self._sites)
        for image in itertools.product(xrange(nxmin, nxmax+1), xrange(nymin, nymax+1), xrange(nzmin, nzmax+1)):
            for site in unit_cell_sites:
                fcoords = site.frac_coords + np.array(image)
                coords = self.lattice.get_cartesian_coords(fcoords)
                submat = [coords] * n
                dists = (site_coords - submat) ** 2
                dists = np.sqrt(dists.sum(axis = 1))
                withindists = (dists <= r) * (dists > 1e-8)
                for i in xrange(n):
                    if withindists[i]:
                        neighbors[i].append((PeriodicSite(site.species_and_occu,fcoords, site.lattice), dists[i]))
                
        return neighbors
    
    
    def get_neighbors_in_shell(self, origin, r, dr):
        """
        Returns all sites in a shell centered on origin (coords) between radii r-dr and r+dr.
        
        """
        outer = self.get_sites_in_sphere(origin, r + dr)
        inner = r - dr
        return [(site, dist) for (site, dist) in outer if dist > inner]
    
    @property
    def formula(self):
        """
        Returns the formula.
        """
        return self.composition.formula
        
    @property
    def composition(self):
        """
        Returns the composition
        """
        elmap = dict()
        for site in self._sites:
            for species, occu in site.species_and_occu.items():
                if species in elmap:
                    elmap[species] += occu
                else:
                    elmap[species] = occu
        return Composition(elmap)

    def get_sorted_structure(self):
        """
        Get a sorted copy of the structure.
        Sites are sorted by the electronegativity of the species.
        """
        sortedsites = sorted(self.sites)
        return Structure(self._lattice,[site.species_and_occu for site in sortedsites],[site.frac_coords for site in sortedsites])

    def interpolate(self,end_structure,numImages=10):
        '''
        Interpolate between this structure and end_structure.
        Returns a list of interpolated structures.
        Defaults to 10 images.
        Added by: Shyue Ping Ong
        Date: Aug 23 2010
        '''
        #Check length of structures
        if len(self) != len(end_structure):
            raise ValueError("You are interpolating structures with different lengths!")
        
        #Check that both structures have the same lattice
        if not ((abs(self.lattice.matrix-end_structure.lattice.matrix) < 0.01)).all():
            raise ValueError("You are interpolating structures with different lattices!")
        
        #Check that both structures have the same species
        for i in xrange(0,len(self)):
            if self[i].species_and_occu != end_structure[i].species_and_occu:
                raise ValueError("You are interpolating different structures!\nStructure 1:\n" + str(self) + "\nStructure 2\n"+str(end_structure))
        
        start_coords = np.array(self.frac_coords)
        end_coords = np.array(end_structure.frac_coords)

        jimage=-np.array(np.around(end_coords-start_coords),int)
        vec = end_coords - start_coords + jimage
        intStructs = [Structure(self.lattice,[site.species_and_occu for site in self._sites],start_coords + float(x)/float(numImages) * vec) for x in xrange(0,numImages+1)]
        return intStructs;
    
    @property
    def is_ordered(self):
        """
        Checks if structure is ordered, meaning no partial occupancies in any of the sites. 
        """
        for site in self._sites:
            if not site.is_ordered:
                return False
        return True

    def __repr__(self):
        outs = []
        outs.append("Structure Summary")
        outs.append(repr(self.lattice))
        for s in self.sites:
            outs.append(repr(s))
        return "\n".join(outs)

    def __str__(self):
        output = str(self.lattice) + "\n"
        output += '\n'.join(str(site) for site in self)
        return output
    
    @property
    def to_dict(self):
        d = {}
        d['lattice'] = self._lattice.to_dict
        d['sites'] = [site.to_dict for site in self]
        return d
    
    @staticmethod
    def from_dict(structure_dict):
        lattice = Lattice(structure_dict['lattice']['matrix'])
        species = []
        coords = []
            
        for site_dict in structure_dict['sites']:
            sp = site_dict['species'] 
            species.append({ Specie(sp['element'], sp['oxidation_state']) if 'oxidation_state' in sp else Element(sp['element'])  : sp['occu'] for sp in site_dict['species']} )
            coords.append(site_dict['abc'])
        return Structure(lattice, species, coords)

class StructureError(Exception):
    
    '''
    Exception class for Structure.
    Raised when the structure has problems, e.g., atoms that are too close.
    '''

    def __init__(self, msg):
        self.msg = msg

    def __str__(self):
        return "Structure Error : " + self.msg

class Composition (collections.Mapping, collections.Hashable):
    """
    Represents a Composition, which is essentially a {element:amount} dictionary.
    
    Works almost completely like a standard python dictionary, except that __getitem__
    is overridden to return 0 when an element is not found.
    
    Also adds more convenience methods relevant to compositions, e.g., get_fraction.
    
    >>> comp = Composition.from_formula("LiFePO4")
    >>> comp.get_atomic_fraction("Li")
    0.0
    >>> comp.get_atomic_fraction(Element("Li"))
    0.14285714285714285
    >>> comp.num_atoms
    7.0
    >>> comp.reduced_formula
    'LiFePO4'
    >>> comp.formula
    'Li1 Fe1 P1 O4'
    >>> comp.get_wt_fraction(Element("Li"))
    0.04399794666951898
    >>> comp.num_atoms
    7.0
    """
    
    """
    Tolerance in distinguishing different composition amounts.
    1e-8 is fairly tight, but should cut out most floating point arithmetic errors.
    """
    amount_tolerance = 1e-8
    
    
    """
    Special formula handling for peroxides and certain elements. This is so that 
    formula output does not write LiO instead of Li2O2 for example.
    """
    special_formulas = {'LiO':'Li2O2','NaO':'Na2O2','KO':'K2O2','HO':'H2O2', 'O':'O2','F':'F2','N':'N2','Cl':'Cl2', 'H':'H2'}
    
    def __init__(self,elmap):
        if any([e < 0 for e in elmap.values()]):
            raise ValueError("Amounts in Composition cannot be negative!")
        if not any([isinstance(e, (Element,Specie)) for e in elmap.keys()]):
            raise TypeError("Keys must be instances of Element!")
        self._elmap = elmap.copy()
        self._natoms = sum(self._elmap.values())

    @property
    def is_element(self):
        '''
        Returns true if composition is for an element
        '''
        return len(self._elmap)==1

    def __getitem__(self,el):
        '''
        Get the amount for element
        '''
        return self._elmap.get(el, 0)
        
    def __eq__(self, other):
        for el in self.elements:
            if self[el] != other[el]:
                return False
        for el in other.elements:
            if self[el] != other[el]:
                return False
        return True

    def __ne__(self, other):
        return not self.__eq__(other)
    
    def __add__(self, other):
        """
        Adds two compositions.
        For example, an Fe2O3 composition + an FeO composition gives a Fe3O4 composition.
        """
        new_el_map = {el:self[el] for el in self}
        for k in other.keys():
            el = smart_element_or_specie(k)
            if el in self:
                new_el_map[el] += other[k]
            else:
                new_el_map[el] = other[k]
        return Composition(new_el_map)
    
    def __sub__(self, other):
        """
        Subtracts two compositions.
        For example, an Fe2O3 composition - an FeO composition gives an FeO2 composition.
        raises a ValueError if the subtracted composition is greater than the original composition
        in any of its elements.
        """
        new_el_map = {el:self[el] for el in self}
        for k in other.keys():
            el = smart_element_or_specie(k)
            if el in self and other[k] <= self[el]:
                new_el_map[el] -= other[k]
            else:
                raise ValueError("All elements in subtracted composition must exist in original composition in equal or lesser amount!")
        return Composition(new_el_map)
    
    def __mul__(self, other):
        """
        Multiply a Composition by an integer or a float.
        Fe2O3 * 4 -> Fe8O12
        """
        if not (isinstance(other, int) or isinstance(other, float)):
            raise ValueError("Multiplication of composition can only be done for integers or floats!")        
        return Composition({el:self[el]*other for el in self})

    def __hash__(self):
        '''
        Minimally effective hash function that just distinguishes between Compositions with different elements.
        '''
        hashcode = 0
        for el in self._elmap.keys():
            #Ignore elements with zero amounts.
            if self[el] > self.amount_tolerance:
                hashcode += el.Z 
        return hashcode
    
    def __contains__(self,el):
        return el in self._elmap
    
    def __len__(self):
        return len(self._elmap)
    
    def __iter__(self):
        return self._elmap.__iter__()
    
    def copy(self):
        return Composition(self._elmap)

    @property
    def formula(self):
        '''
        Returns a formula string, e.g. Li4 Fe4 P4 O16.
        '''
        elements = self._elmap.keys()
        elements = sorted(elements,key=lambda el: el.X)
        formula = []
        for el in elements:
            if self[el] != 0:
                formula.append(el.symbol+formula_double_format(self[el], False))
        return ' '.join(formula)
    
    def get_reduced_formula_and_factor(self):
        '''
        Returns a pretty normalized formula and a multiplicative factor, i.e., Li4Fe4P4O16 returns (LiFePO4, 4).
        '''
        
        elements = self._elmap.keys()
        elements = sorted(elements,key=lambda el: el.X)
        elements = filter(lambda el: self[el] != 0, elements)
        num_el = len(elements)
        contains_polyanion = False
        if num_el >= 3:
            contains_polyanion = (elements[num_el-1].X - elements[num_el-2].X < 1.65)
        
        factor = reduce(gcd,self._elmap.values())

        reduced_form = ''
        n = num_el
        if contains_polyanion:
            n -= 2
        
        for i in xrange(0,n):
            el = elements[i]
            normamt = self._elmap[el] *1.0 / factor
            reduced_form += el.symbol + formula_double_format(normamt) 

        if contains_polyanion:
            polyamounts = list()
            polyamounts.append(self._elmap[elements[num_el - 2]] / factor)
            polyamounts.append(self._elmap[elements[num_el - 1]] / factor)
            polyfactor = reduce(gcd,polyamounts)
            for i in xrange(n,num_el):
                el = elements[i]
                normamt = self._elmap[el] / factor / polyfactor
                if normamt != 1.0:
                    if normamt != int(normamt):
                        polyfactor = 1;
                        break;
            
            poly_form = ""
            
            for i in xrange(n,num_el):
                el = elements[i]
                normamt = self._elmap[el] / factor / polyfactor
                poly_form += el.symbol + formula_double_format(normamt);

            if polyfactor != 1:
                reduced_form += "("+poly_form+ ")"+ str(int(polyfactor))
            else:
                reduced_form += poly_form

        
        if reduced_form in Composition.special_formulas:
            reduced_form = Composition.special_formulas[reduced_form]
            factor = factor / 2

        return (reduced_form, factor)

    @property
    def reduced_formula(self):
        '''
        Returns a pretty normalized formula, i.e., LiFePO4 instead of Li4Fe4P4O16.
        '''
        return self.get_reduced_formula_and_factor()[0]

    @property
    def elements(self):
        '''
        Returns view of elements in Composition.
        '''
        return self._elmap.keys()

    def __str__(self):
        return self.formula

    @property
    def num_atoms(self):
        '''
        Total number of atoms in Composition
        '''
        return self._natoms
    
    @property
    def weight(self):
        '''
        Total molecular weight of Composition
        '''
        return sum([amount * el.atomic_mass for el, amount in self._elmap.items()])
    
    def get_atomic_fraction(self, el):
        '''
        Arguments:
            el - Element
        Returns:
            Atomic fraction for element el in Composition
        '''
        return self[el]/self._natoms
    
    def get_wt_fraction(self, el):
        '''
        Arguments:
            el - Element
        Returns:
            Weight fraction for element el in Composition
        '''
        return el.atomic_mass*self[el]/self.weight
   
    @staticmethod
    def from_formula(formula):
        '''
        Arguments:
            formula - A string formula, e.g. Fe2O3, Li3Fe2(PO4)3
        Returns:
            Composition with that formula.
        '''
        def get_sym_dict(f, factor):
            sym_dict = {}
            for m in re.finditer(r"([A-Z][a-z]*)([\.\d]*)", f):
                el = m.group(1)
                amt = 1
                if m.group(2).strip() != "":
                    amt = float(m.group(2))
                if el in sym_dict:
                    sym_dict[el] += amt * factor
                else:
                    sym_dict[el] = amt * factor
            return sym_dict
        m = re.search(r"\(([^\(\)]+)\)([\.\d]*)", formula)
        if m:
            factor = 1
            if m.group(2) != "":
                factor = float(m.group(2))
            unit_sym_dict = get_sym_dict(m.group(1), factor)
            expanded_formula = formula.replace(m.group(), "".join([el+str(amt) for el, amt in unit_sym_dict.items()]))
            return Composition.from_formula(expanded_formula)
        return Composition.from_dict(get_sym_dict(formula, 1))
    
    
    def __repr__(self):
        return "Comp: " + self.formula
        
    @staticmethod
    def from_dict(sym_dict):
        '''
        Arguments:
            sym_dict - A element symbol: amount dict, e.g. {"Fe":2, "O":3}
        Returns:
            Composition with that formula.
        '''
        return Composition( {Element(sym) : amt for sym, amt in sym_dict.items()})
    
    @property
    def to_dict(self):
        '''
        Returns:
            dict with element symbol and (unreduced) amount e.g. {"Fe": 4.0, "O":6.0}
        '''
        return {e.symbol: a for e, a in self.items()}
    
    @property
    def to_reduced_dict(self):
        '''
        Returns:
            dict with element symbol and reduced amount e.g. {"Fe": 2.0, "O":3.0}
        '''
        reduced_formula = self.reduced_formula
        c = Composition.from_formula(reduced_formula)
        return c.to_dict




#!/usr/bin/env python

'''
Interface with command line qhull.
Right now only tested on Linux systems.
'''

from __future__ import division

__author__="Shyue Ping Ong"
__copyright__ = "Copyright 2011, The Materials Project"
__version__ = "1.0"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__status__ = "Production"
__date__ ="$Sep 23, 2011M$"

import subprocess
import re

def run_qhull_command(command, data, proc_command = int, output_skip=1):
    """
    Helper function for actual qconvex and qvoronoi and qvertex commands.
    """
    prep_str = str(len(data[0])) + "\n"
    prep_str += str(len(data)) +"\n"
    prep_str += "\n".join([' '.join([str(i) for i in row]) for row in data])
    p = subprocess.Popen(command,stdout=subprocess.PIPE,stdin=subprocess.PIPE)
    output = p.communicate(input=prep_str)[0]
    output = re.split("\n", output)
    for i in xrange(output_skip):
        output.pop(0)
    results = list()
    for row in output:
        cleanrow = row.strip()
        if cleanrow != "":
            results.append([proc_command(i) for i in re.split("\s+",cleanrow)])
    return results

def qconvex(data):
    """
    Input data should be in the form of a list of a list of floats.
    Returns the facets of the convex hull as a list of a list of integers.
    """
    return run_qhull_command(['qconvex','i','Qt'], data, int, 1)

def qvoronoi(data):
    """
    Input data should be in the form of a list of a list of floats.
    Returns voronoi results as a list of a list of integers.
    """
    return run_qhull_command(['qvoronoi','Fv'], data, int, 1)

def qvertex(data):
    """
    Input data should be in the form of a list of a list of floats.
    Returns the facets of voronoi construction as a list of a list of float.
    """
    return run_qhull_command(['qvoronoi','p'], data, float, 2)




#!/usr/bin/env python

"""
This module provides utility classes for io operations.
"""

__author__="Shyue Ping Ong"
__copyright__ = "Copyright 2011, The Materials Project"
__version__ = "1.0"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__status__ = "Production"
__date__ ="$Sep 23, 2011M$"

import gzip
import bz2
import re
import numpy

def file_open_zip_aware(filename, *args):
    """
    This wrapper wraps around the bz2, gzip and standard python file open
    to deal intelligently with bzipped, gzipped or standard text files.
    """
    if filename.split(".")[-1].upper() == "BZ2":
        return bz2.BZ2File(filename,*args)
    elif filename.split(".")[-1].upper() == "GZ" or filename.split(".")[-1] == "z":
        return gzip.GzipFile(filename,*args)
    else:
        return open(filename, *args)

def clean_lines(string_list, remove_empty_lines = True):
    """
    type(list) = list of strings
    # Strips whitespace from strings '\n' and '\r' and empty lines from a list
    # (the list is usually obtained with the ".readlines()" method)
    """
    
    for s in string_list:
        clean_s = s
        if '#' in s:
            ind = s.index('#')
            clean_s = s[:ind]
        clean_s = clean_s.strip()
        if (not remove_empty_lines) or clean_s != '':
            yield clean_s
                    
# @author Rickard Armiento
def micro_pyawk(filename, search, results=None, debug=None, postdebug=None):
    """
    Small awk-mimicking search routine.
       
    'file' is file to search through.
    'search' is the "search program", a list of lists/tuples with 3 elements; i.e.
      [[regex,test,run],[regex,test,run],...]
    'results' is a an object that your search program will have access to for storing results.

    Here regex is either as a Regex object, or a string that we compile into a Regex.
    test and run are callable objects.

    This function goes through each line in filename, and if regex matches that line *and* 
    test(results,line)==True (or test == None) we execute run(results,match), 
    where match is the match object from running Regex.match.   
    
    The default results is an empty dictionary. Passing a results object let you interact 
    with it in run() and test(). Hence, in many occasions it is thus clever to use results=self. 
    
    Returns: results
    """
    if results == None: results = {}

    # Compile strings into regexs
    for entry in search:
        if isinstance(entry[0], str):
            entry[0] = re.compile(entry[0])
            
    reader = file_open_zip_aware(filename)
    for line in reader:
        for i in range(len(search)):
            match = search[i][0].search(line)
            if match and (search[i][1] == None or search[i][1](results,line)):
                if debug != None:
                    debug(results,match)
                search[i][2](results,match)
                if postdebug != None:
                    postdebug(results,match)

    reader.close()
    return results

def clean_json(input_json):
    """
    This method cleans an input json-like object, either a list or a dictionary, nested or otherwise,
    by converting all non-string dictionary keys (such as int and float) to strings.
    """
    if isinstance(input_json, (list, numpy.ndarray)):
        return [clean_json(i) for i in input_json]
    elif isinstance(input_json, dict):
        return {str(k): clean_json(v) for k, v in input_json.items()}
    elif isinstance(input_json, (int, float)):
        return input_json
    elif isinstance(input_json, basestring):
        return str(input_json)
    else:
        return clean_json(input_json.to_dict)
